@@ -66,6 +66,7 @@ static int lc_process_conf_apache_include(const char *pathname, const char *path
 			if (lcpcafret < 0) {
 				retval = -1;
 				/* XXX: should we break here (abort further including of files from a directory if one fails ?) */
+				break;
 			}
 		}
 
@@ -81,7 +82,8 @@ static int lc_process_conf_apache_include(const char *pathname, const char *path
 }
 
 static int lc_process_conf_apache_file(const char *configfile, const char *pathprefix) {
-	FILE *configfp = NULL;
+	LC_FILE *configfp = NULL;
+	const char *local_lc_errfile;
 	char linebuf[LC_LINEBUF_LEN] = {0}, *linebuf_ptr = NULL, *tmp_ptr = NULL;
 	char *lastsection = NULL;
 	char qualifbuf[LC_LINEBUF_LEN] = {0};
@@ -89,6 +91,7 @@ static int lc_process_conf_apache_file(const char *configfile, const char *pathp
 	char *fgetsret = NULL;
 	int lcpvret = -1, lpcafret = -1;
 	int invalid_section = 0, ignore_section = 0;
+	int local_lc_errline;
 	int retval = 0;
 	lc_err_t save_lc_errno = LC_ERR_NONE;
 
@@ -97,7 +100,12 @@ static int lc_process_conf_apache_file(const char *configfile, const char *pathp
 		strncpy(qualifbuf, pathprefix, sizeof(qualifbuf) - 1);
 	}
 
+	local_lc_errfile = configfile;
+	local_lc_errline = 0;
+
 	if (configfile == NULL) {
+		lc_errfile = local_lc_errfile;
+		lc_errline = local_lc_errline;
 		lc_errno = LC_ERR_INVDATA;
 		return(-1);
 	}
@@ -105,19 +113,23 @@ static int lc_process_conf_apache_file(const char *configfile, const char *pathp
 	configfp = lc_fopen(configfile, "r");
 
 	if (configfp == NULL) {
+		lc_errfile = local_lc_errfile;
+		lc_errline = local_lc_errline;
 		lc_errno = LC_ERR_CANTOPEN;
 		return(-1);
 	}
 
 	while (1) {
-		fgetsret = fgets(linebuf, sizeof(linebuf) - 1, configfp);
+		fgetsret = lc_fgets(linebuf, sizeof(linebuf) - 1, configfp);
 		if (fgetsret == NULL) {
 			break;
 		}
-		if (feof(configfp)) {
+		if (lc_feof(configfp)) {
 			break;
 		}
 
+		local_lc_errline++;
+
 		/* Remove trailing crap (but not spaces). */
 		linebuf_ptr = &linebuf[strlen(linebuf) - 1];
 		while (*linebuf_ptr < ' ' && linebuf_ptr >= linebuf) {
@@ -127,19 +139,19 @@ static int lc_process_conf_apache_file(const char *configfile, const char *pathp
 
 		/* Remove leading spaces. */
 		linebuf_ptr = &linebuf[0];
-		while (*linebuf_ptr == ' ' || *linebuf_ptr == '\t') {
+		while ((*linebuf_ptr == ' ' || *linebuf_ptr == '\t') && linebuf_ptr < (linebuf + sizeof(linebuf))) {
 			linebuf_ptr++;
 		}
 
 		/* Handle section header. */
-		if (linebuf_ptr[0] == '<' && linebuf_ptr[strlen(linebuf_ptr) - 1] == '>') {
+		if (linebuf_ptr[0] == '<' && linebuf_ptr[strlen(linebuf_ptr) - 1] == '>' && linebuf_ptr < (linebuf + sizeof(linebuf))) {
 			/* Remove < and > from around the data. */
 			linebuf_ptr[strlen(linebuf_ptr) - 1] = '\0';
 			linebuf_ptr++;
 
 			/* Lowercase the command part of the section. */
 			tmp_ptr = linebuf_ptr;
-			while (*tmp_ptr != '\0' && *tmp_ptr != ' ') {
+			while (*tmp_ptr != '\0' && *tmp_ptr != ' ' && tmp_ptr < (linebuf + sizeof(linebuf))) {
 				*tmp_ptr = tolower(*tmp_ptr);
 				tmp_ptr++;
 			}
@@ -164,6 +176,8 @@ static int lc_process_conf_apache_file(const char *configfile, const char *pathp
 					fprintf(stderr, "Last opened = \"%s\", Closing = \"%s\"\n", lastsection, cmd);
 #endif
 					retval = -1;
+					lc_errfile = local_lc_errfile;
+					lc_errline = local_lc_errline;
 					lc_errno = LC_ERR_BADFORMAT;
 
 					/* For this error, we abort immediately. */
@@ -232,6 +246,8 @@ static int lc_process_conf_apache_file(const char *configfile, const char *pathp
 				fprintf(stderr, "Invalid section: \"%s\"\n", qualifbuf);
 #endif
 				invalid_section = 1;
+				lc_errfile = local_lc_errfile;
+				lc_errline = local_lc_errline;
 				lc_errno = LC_ERR_INVSECTION;
 				retval = -1;
 			}
@@ -267,7 +283,7 @@ static int lc_process_conf_apache_file(const char *configfile, const char *pathp
 
 			/* Delete space at the end of the command. */
 			cmdend--; /* It currently derefs to the seperator.. */
-			while (*cmdend <= ' ') {
+			while (*cmdend <= ' ' && cmdend >= sep) {
 				*cmdend = '\0';
 				cmdend--;
 			}
@@ -288,6 +304,8 @@ static int lc_process_conf_apache_file(const char *configfile, const char *pathp
 		/* Handle special commands. */
 		if (strcasecmp(cmd, "include") == 0) {
 			if (value == NULL) {
+				lc_errfile = local_lc_errfile;
+				lc_errline = local_lc_errline;
 				lc_errno = LC_ERR_BADFORMAT;
 				retval = -1;
 #ifdef DEBUG
@@ -321,13 +339,18 @@ static int lc_process_conf_apache_file(const char *configfile, const char *pathp
 #ifdef DEBUG
 				fprintf(stderr, "Invalid command: \"%s\"\n", cmd);
 #endif
+				lc_errfile = local_lc_errfile;
+				lc_errline = local_lc_errline;
 				lc_errno = LC_ERR_INVCMD;
 			} else {
 #ifdef DEBUG
 				fprintf(stderr, "Error processing command (command was valid, but an error occured, errno was set)\n");
 #endif
 			}
+			lc_errfile = local_lc_errfile;
+			lc_errline = local_lc_errline;
 			retval = -1;
+			break;
 		} else {
 			lc_errno = save_lc_errno;
 		}
@@ -340,7 +363,7 @@ static int lc_process_conf_apache_file(const char *configfile, const char *pathp
 		*tmp_ptr = '\0';
 	}
 
-	fclose(configfp);
+	lc_fclose(configfp);
 
 	return(retval);
 }
