@@ -46,6 +46,7 @@ const char *lc_err_usererrmsg = NULL;
 const char *lc_errfile = NULL;
 int lc_optind = 0;
 int lc_errline = 0;
+char *lc_erroptname =  NULL;
 
 extern char **environ;
 
@@ -86,6 +87,8 @@ static int lc_process_var_hostname4(uint32_t *data, const char *value, const cha
 }
 
 static int lc_process_var_hostname6(void *data, const char *value, const char **endptr) {
+	lc_errno = LC_ERR_BADFORMAT;
+
 	return(-1);
 }
 
@@ -137,10 +140,16 @@ static int lc_process_var_ip4(uint32_t *data, const char *value, const char **en
 		*data = ipval;
 	}
 
+	if (retval < 0) {
+		lc_errno = LC_ERR_BADFORMAT;
+	}
+
 	return(retval);
 }
 
 static int lc_process_var_ip6(void *data, const char *value, const char **endptr) {
+	lc_errno = LC_ERR_BADFORMAT;
+
 	return(-1);
 }
 
@@ -157,6 +166,8 @@ static int lc_process_var_addr4(uint32_t *data, const char *value, const char **
 		return(lc_pv_ret);
 	}
 
+	lc_errno = LC_ERR_BADFORMAT;
+
 	return(-1);
 }
 
@@ -629,6 +640,7 @@ static int lc_process_cmdline(int argc, char **argv) {
 	if (newargv == NULL) {
 		lc_errfile = local_lc_errfile;
 		lc_errline = local_lc_errline;
+		lc_erroptname = NULL;
 		lc_errno = LC_ERR_ENOMEM;
 		return(-1);
 	}
@@ -640,6 +652,7 @@ static int lc_process_cmdline(int argc, char **argv) {
 	if (usedargv == NULL) {
 		lc_errfile = local_lc_errfile;
 		lc_errline = local_lc_errline;
+		lc_erroptname = NULL;
 		lc_errno = LC_ERR_ENOMEM;
 		free(newargv);
 		return(-1);
@@ -724,6 +737,7 @@ static int lc_process_cmdline(int argc, char **argv) {
 						fprintf(stderr, "Argument required.\n");
 						lc_errfile = local_lc_errfile;
 						lc_errline = local_lc_errline;
+						lc_erroptname = strdup(cmdarg);
 						lc_errno = LC_ERR_BADFORMAT;
 						free(usedargv);
 						free(newargv);
@@ -736,6 +750,9 @@ static int lc_process_cmdline(int argc, char **argv) {
 
 				chkretval = lc_handle(handler, handler->var, NULL, cmdoptarg, LC_FLAGS_CMDLINE);
 				if (chkretval < 0) {
+					lc_errfile = local_lc_errfile;
+					lc_errline = local_lc_errline;
+					lc_erroptname = strdup(cmdarg);
 					retval = -1;
 				}
 
@@ -746,6 +763,7 @@ static int lc_process_cmdline(int argc, char **argv) {
 				fprintf(stderr, "Unknown option: --%s\n", cmdarg);
 				lc_errfile = local_lc_errfile;
 				lc_errline = local_lc_errline;
+				lc_erroptname = strdup(cmdarg);
 				lc_errno = LC_ERR_INVCMD;
 				free(usedargv);
 				free(newargv);
@@ -776,6 +794,7 @@ static int lc_process_cmdline(int argc, char **argv) {
 							fprintf(stderr, "Argument required.\n");
 							lc_errfile = local_lc_errfile;
 							lc_errline = local_lc_errline;
+							lc_erroptname = strdup(cmdarg);
 							lc_errno = LC_ERR_BADFORMAT;
 							free(usedargv);
 							free(newargv);
@@ -790,6 +809,7 @@ static int lc_process_cmdline(int argc, char **argv) {
 					if (chkretval < 0) {
 						lc_errfile = local_lc_errfile;
 						lc_errline = local_lc_errline;
+						lc_erroptname = strdup(cmdarg);
 						retval = -1;
 					}
 
@@ -800,6 +820,9 @@ static int lc_process_cmdline(int argc, char **argv) {
 					fprintf(stderr, "Unknown option: -%c\n", ch);
 					lc_errfile = local_lc_errfile;
 					lc_errline = local_lc_errline;
+					lc_erroptname = malloc(2);
+					lc_erroptname[0] = ch;
+					lc_erroptname[1] = '\0';
 					lc_errno = LC_ERR_INVCMD;
 					free(usedargv);
 					free(newargv);
@@ -873,6 +896,7 @@ int lc_process_var(const char *var, const char *varargs, const char *value, lc_f
 		    handler->type != LC_VAR_SECTIONSTART &&
 		    handler->type != LC_VAR_SECTIONEND) {
 			lc_errno = LC_ERR_BADFORMAT;
+			lc_erroptname = NULL;
 			break;
 		}
 
@@ -959,6 +983,7 @@ int lc_process_file(const char *appname, const char *pathname, lc_conf_type_t ty
 		default:
 			chkretval = -1;
 			lc_errno = LC_ERR_INVDATA;
+			lc_erroptname = NULL;
 			break;
 	}
 
@@ -1053,6 +1078,11 @@ void lc_cleanup(void) {
 		lc_err_usererrmsg = NULL;
 	}
 
+	if (lc_erroptname) {
+		free(lc_erroptname);
+		lc_erroptname = NULL;
+	}
+
 	varhandlers = NULL;
 
 	return;
@@ -1098,6 +1128,7 @@ void lc_seterrstr(const char *usererrmsg) {
 char *lc_geterrstr(void) {
 	static char retval[512];
 	const char *errmsg = NULL;
+	const char *local_lc_errfile;
 
 	switch (lc_errno) {
 		case LC_ERR_NONE:
@@ -1139,9 +1170,15 @@ char *lc_geterrstr(void) {
 	}
 
 	if (lc_errfile == NULL) {
-		snprintf(retval, sizeof(retval), "%s:%i: %s", "<no file>", lc_errline, errmsg);
+		local_lc_errfile = "<no file>";
 	} else {
-		snprintf(retval, sizeof(retval), "%s:%i: %s", lc_errfile, lc_errline, errmsg);
+		local_lc_errfile = lc_errfile;
+	}
+
+	if (lc_erroptname != NULL) {
+		snprintf(retval, sizeof(retval), "%s:%i: \"%s\": %s", local_lc_errfile, lc_errline, lc_erroptname, errmsg);
+	} else {
+		snprintf(retval, sizeof(retval), "%s:%i: %s", local_lc_errfile, lc_errline, errmsg);
 	}
 
 	retval[sizeof(retval) - 1] = '\0';
